/*
 * $Id: readvcs.java,v 1 2002/05/24 11:02:30
 *
 * Transmorpher
 *
 * Copyright (C) 2001-2002 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package fr.fluxmedia.tmcontrib.generator;

// Imported TRANSMORPHER Classes
import fr.fluxmedia.transmorpher.TProcessFactory.TReader;
import fr.fluxmedia.transmorpher.TProcessFactory.XML_Port;
import fr.fluxmedia.transmorpher.Utils.Parameters;
import fr.fluxmedia.transmorpher.Utils.StringParameters;
//Imported JAVA classes
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.*;
// Imported SAX Classes
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

/**
 *  Read a vcal or vcard (specify by the type parameter) file which coordinates (local path or URL) are given in the
 * file parameter of the generate call, parse it, process it through vcsReader
 * and generate a SAX event flow to the next handler
 *
 *@author    Bruno Charre
 */
public class readvcs extends TReader {

	//vcsReader reader = null;
	/** name of the file to parse*/
	String iFilename = null;

	/** Is the file a vCard ? */
	boolean card = true;


	/**
	 * Constructor for the readvcs object.
	 *
	 *@param  pOut               out port of this generator
	 *@param  pParam             parameters of this generator
	 *@param  pStaticAttributes  Description of the Parameter
	 *@exception  SAXException   Description of the Exception
	 *@exception  IOException    Description of the Exception
	 */
	public readvcs(String[] pOut, Parameters pParam, StringParameters pStaticAttributes) throws SAXException, IOException {
		super(pOut, pParam, pStaticAttributes);
		if (pParam.getParameter("type")!=null){
			String pCard = (String) pParam.getParameter("type");
			if (pCard.equals("vcard")) {
				card = true;
			} else {
				card = false;
			}
		}
		iReader = new vcsReader(card);
	}

	/**
	 * Starts to parse the document
	 *
	 *@exception  IOException   Description of the Exception
	 *@exception  SAXException  Description of the Exception
	 */
	public void execProcess() throws IOException, SAXException {
		iFilename = (String) getParameter("file");
		if (isThread) {
			myThread.start();
		} else {
			read();
		}//end if
	}//end proc

	/**
	 * Creates an input source for the parser with the name of the file and starts
	 * the parsing
	 *
	 *@exception  IOException   Description of the Exception
	 *@exception  SAXException  Description of the Exception
	 */
	public void read() throws IOException, SAXException {
		InputSource inSource;
		if (iFilename != null) {
			if (iFilename.startsWith("http://")) {
				URL url = new URL(iFilename);
				URLConnection urlConnect = url.openConnection();
				InputStream inStream = urlConnect.getInputStream();
				inSource = new InputSource(inStream);
				inSource.setSystemId(iFilename);
				iReader.parse(inSource);
			} else {
				File file = new File(iFilename);
				inSource = new InputSource(new FileInputStream(file));
				inSource.setSystemId(file.toURL().toExternalForm());
				iReader.parse(inSource);
			}
		} else {
			//reader.parse(standardInput);
		}//end if
	}

	/**
	 *  Begins the parsing of a file. 
	 * This method is called inside a repeat . At each loop, a new file is parsed.
	 *
	 *@param  file              name of the file to parse
	 *@exception  IOException   Description of the Exception
	 *@exception  SAXException  Description of the Exception
	 */
	public void execProcess(String file) throws IOException, SAXException {
		iFilename = file;
		try {
			if (iFilename != null) {
				read();
				//iReader.parse(iFilename);
			} else {
				//reader.parse(standardInput);
			}//end if
		} catch (SAXException e) {
			System.out.println("[readvcs] " + e + " " + file);
			e.printStackTrace();
		}
	}

	/* public void generatePort(){
	 *iListOut[0] = new XML_Port(nameOut[0],this,(XMLReader)iReader,0,XML_Port.OUT);
	 *}  */
	 
	/** 
	* Starts the parsing of a file using Thread 
	*/
	public final void run() {
		if (!isRunning) {
			try {
				read();
				//myThread.interrupt();
			} catch (Exception e) {
				System.err.println("Fin du parse 2 !erreur dans le thread");
				e.printStackTrace();
			}//end try
			isRunning = true;
		}//end if
	}//end run
}

