/**
 * $Id: tidy.java,v 1.4 2002/09/24 05:59:37 triolet Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001-2002 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package fr.fluxmedia.tmcontrib.generator;
// Imported Transmorpher classes
import fr.fluxmedia.transmorpher.TProcessFactory.*;
import fr.fluxmedia.transmorpher.Files.ReadWrite;
import fr.fluxmedia.transmorpher.Utils.Parameters;
import fr.fluxmedia.transmorpher.Utils.StringParameters;
import fr.fluxmedia.transmorpher.Utils.SystemResources;
// Imported Java classes
import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.InputStream;
import java.io.FileInputStream;
import java.lang.Runnable;
import java.net.URL;
import java.net.URLConnection;
import java.util.Hashtable;
import javax.xml.parsers.SAXParserFactory;
// Imported JAXP classes
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.sax.SAXResult;
// Imported DOM classes
import org.w3c.dom.Node;
// Imported Tidy class
import org.w3c.tidy.Tidy;
// Imported SAX classes
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * read a HTML file which coordinates (local path or URL) is give in the
 * file parameter of the generate call, parse it, process it through TIDY
 * and generate a SAX event flow to the next handler
 *
 *@author Jrme.Euzenat@inrialpes.fr
 *@author Fabien.Triolet@inrialpes.fr
 */

public final class tidy extends TReader {

	/** The file to read */
	String iFilename = null;
	/** the standard input to parse by default*/
	InputSource standardInput = null;

	/**
	 *Constructor for the tidy object
	 *
	 *@param  pOut               Description of the Parameter
	 *@param  pParam             Description of the Parameter
	 *@param  pStaticAttributes  Description of the Parameter
	 *@exception  SAXException   Description of the Exception
	 *@exception  IOException    Description of the Exception
	 */
	public tidy(String[] pOut, Parameters pParam, StringParameters pStaticAttributes) throws SAXException, IOException {

		super(pOut, pParam, pStaticAttributes);
		try {
			SAXParserFactory parserFactory = SAXParserFactory.newInstance();

			parserFactory.setFeature(NAMESPACES_FEATURE_ID, DEFAULT_NAMESPACES);
			parserFactory.setFeature(NAMESPACES_PREFIXES_FEATURE_ID, DEFAULT_NAMESPACES_PREFIXES);
			parserFactory.setFeature(VALIDATION_FEATURE_ID, DEFAULT_VALIDATION);
			parserFactory.setFeature(SCHEMA_VALIDATION_FEATURE_ID, DEFAULT_SCHEMA_VALIDATION);

			iReader = (parserFactory.newSAXParser()).getXMLReader();
			//System.out.println(iReader);
		} catch (Exception e) {
			System.out.println("[tidy]Exception :" + e);
			System.out.println("[tidy] " + SystemResources.getProperty("XML_READER"));
		}//end try

	}

	/**
	 * Exec the process
	 *
	 *@exception  IOException   Description of the Exception
	 *@exception  SAXException  Description of the Exception
	 */
	public void execProcess() throws IOException, SAXException {

		iFilename = (String) getParameter("file");
		if (isThread) {
			myThread.start();
		} else {
			read();
		}//end if
	}//end proc

	/**
	 *  Description of the Method
	 *
	 *@exception  IOException   Description of the Exception
	 *@exception  SAXException  Description of the Exception
	 */
	public void read() throws IOException, SAXException {
		Tidy cleaner = new Tidy();
		cleaner.setErrout(new PrintWriter(System.err));
		InputSource inSource;
		InputStream inStream;
		if (iFilename != null) {
			if (iFilename.startsWith("http://")) {
				URL url = new URL(iFilename);
				URLConnection urlConnect = url.openConnection();
				inStream = urlConnect.getInputStream();
			} else {
				File file = new File(iFilename);
				inStream = new FileInputStream(file);
				//inSource.setSystemId(file.toURL().toString());
			}
			Node document = cleaner.parseDOM(inStream, null);
			Transformer transformer = null;
			try {
				transformer = TProcessFactory.getSAXTFactory().newTransformer();
			} 
			catch (Exception e) {}
			DOMSource source = new DOMSource(document);
			source.setSystemId(iFilename);
			ContentHandler handler = iReader.getContentHandler();
			SAXResult result = new SAXResult(handler);
			try {
				transformer.transform(source, result);
			} 
			catch (TransformerException e) {}
		} else {
			iReader.parse(standardInput);
		}//end if
	}

	/**
	 *  Description of the Method
	 *
	 *@param  fileName          Description of the Parameter
	 *@exception  IOException   Description of the Exception
	 *@exception  SAXException  Description of the Exception
	 */
	public void execProcess(String fileName) throws IOException, SAXException { }

	/**  Main processing method for the tidy object */
	public final void run() {
		if (!isRunning) {
			try {
				read();
				isRunning = true;
				//myThread.interrupt();
			} catch (Exception e) {
				System.err.println("Fin du parse 2 !erreur dans le thread" + e);
				e.printStackTrace();
				isRunning = true;
			}//end try
		}//fin if
	}//end run
}

