/*
 * Transmorpher
 *
 * Copyright (C) 2001-2002 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package fr.fluxmedia.tmcontrib.serializer;
// Imported fr.fluxmedia.transmorpher
import fr.fluxmedia.transmorpher.Files.ReadWrite;
import fr.fluxmedia.transmorpher.TProcessFactory.*;
import fr.fluxmedia.transmorpher.Utils.Parameters;
import fr.fluxmedia.transmorpher.Utils.StringParameters;
// Imported java.io classes
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Properties;
// Imported org.apache.fop
import org.apache.fop.apps.Driver;
import org.apache.fop.messaging.MessageHandler;
import org.apache.log.Hierarchy;
import org.apache.log.format.PatternFormatter;
import org.apache.log.Priority;
import org.apache.log.LogTarget;
import org.apache.log.output.io.StreamTarget;
// Imported SAX classes
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;

/**
 * 
 * This class allows transmorpher to produce pdf or ps.
 *  To use this serializer, transmorpher has to know it.
 *  To do this, just add the following declaration of defextern in the process file : 
 * <code><p>&lt defextern name="fop" class="fr.fluxmedia.transmorpher.tmcontrib.serializer.PDFSerializer/&gt</p></code>
 * <p>A fop serializer can now be declared in the process file :
 * <code><p>&lt serialize id="outputPDF" type="fop" in="out1" &gt</p>
 * <p>&lt with-param name="file"&gt ../samples/fo-sample/output/${filename}.pdf &lt /with-param &gt</p>
 *<p>&lt with-param name="format"&gt pdf &lt /with-param &gt</p>			
 *<p>&lt /serialize &gt</p></code></p>
 *<p>Available formats are :
 * <ul>
 * <li>pdf</li>
 * <li>ps</li>
 * </ul></p>
 * <p>
 *  Compiling: following libs are required
 * <ul>
 * <li>fop.jar</li>
 * <li>logkit jar</li>
 * </ul>
 * Running: following libs are required
 * <ul>
 * <li>fop.jar</li>
 * <li>batik.jar</li>
 * <li>avalon-framework-4.0.jar</li>
 * <li>logkit-1.0.jar</li>
 * </ul>
 * </p>
 *@author    triolet
 */
public final class PDFSerializer extends TSerializer {

	/** the serializer */
	private Driver driver = null;

	/** format of the file (default format  is pdf)*/
	private String format="pdf";

	/**
	 * Constructor for the PDFSerializer object
	 *
	 *@param  pIn                in ports
	 *@param  pParam             the parameters of this component
	 *@param  pStaticAttributes  Description of the Parameter
	 *@exception  SAXException   Any SAX exception, possibly wrapping another exception.
	 *@exception  IOException    Description of the Exception
	 */
	public PDFSerializer(String[] pIn, Parameters pParam, StringParameters pStaticAttributes)
			 throws SAXException, IOException {
		super(pIn, pParam, pStaticAttributes);

		driver = new Driver();
		
		MessageHandler.setScreenLogger(new org.apache.avalon.framework.logger.NullLogger());
		driver.setLogger(new org.apache.avalon.framework.logger.NullLogger());

		format = (String) pParam.getParameter("format");
		driver.setRenderer(Driver.RENDER_PDF);
		if ((format.toLowerCase()).equals("pdf")) {
			driver.setRenderer(Driver.RENDER_PDF);
		}
		if ((format.toLowerCase()).equals("ps")) {
			driver.setRenderer(Driver.RENDER_PS);
		}
	}

	/**
	 *  Gets the contentHandler attribute of the PDFSerializer object
	 *
	 *@return    The contentHandler value
	 */
	public ContentHandler getContentHandler() {
		return driver.getContentHandler();
	}

	/**
	 *  Sets the outputStream of this PDFSerializer object
	 *
	 */
	public void setOutputStream() {
		super.setOutputStream();
			if (fileName.lastIndexOf('$')==-1){ // all parameters have been binded
			try {
				if (!format.equals("")) {
					fileName = fileName.substring(0, fileName.lastIndexOf('.') + 1) + format;
				}
				FileOutputStream FOS = null;
				FOS = new FileOutputStream(new File(fileName));				
				driver.setOutputStream(FOS);
				generatePort();
			} catch (IOException IOE) {
				System.err.println("[PDFSerializer]error : " + IOE);
				System.err.println("[PDFSerializer]error : f=" + fileName);
				IOE.printStackTrace();
			} //end try
			catch (Exception e) {
				System.err.println("[PDFSerializer]erreur : " + e);
				e.printStackTrace();
			}//end try
		}
	}

	/**Generates ports for this component */
	public void generatePort() {
		super.generatePort((ContentHandler) getContentHandler());
	}
}

