/*
 * Transmorpher
 *
 * Copyright (C) 2001-2002 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package fr.fluxmedia.tmcontrib.serializer;
// Imported fr.fluxmedia.transmorpher
import fr.fluxmedia.transmorpher.Files.ReadWrite;
import fr.fluxmedia.transmorpher.TProcessFactory.*;
import fr.fluxmedia.transmorpher.Utils.Parameters;
import fr.fluxmedia.transmorpher.Utils.StringParameters;
import java.io.BufferedWriter;
// Imported java.io classes
import java.io.File;
import java.io.FileOutputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.Properties;
// Imported jfor classes
import org.jfor.jfor.converter.Converter;
// Imported SAX classes
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;

/**
 * This class allows transmorpher to produce pdf or ps.
 *  To use this serializer, transmorpher has to know it.
 *  To do this, just add the following declaration of defextern in the process file : 
 * <code><p>&lt defextern name="rtf" class="fr.fluxmedia.transmorpher.tmcontrib.serializer.RTFSerializer/&gt</p></code>
 * <p>A rtf serializer can now be declared in the process file :
 * <code><p>&lt serialize id="outputRTF" type="rtf" in="out1" &gt</p>
 * <p>&lt with-param name="file"&gt ../samples/fo-sample/output/${filename}.rtf &lt /with-param &gt</p>
 *<p>&lt with-param name="format"&gt pdf &lt /with-param &gt</p>			
 *<p>&lt /serialize &gt</p></code></p>
 *<p>Available format is :
 * <ul>
 * <li>rtf</li></ul></p>
 * <p>
 *  Compiling: following jar is required
 * <ul><li>jfor.jar</li></ul></p>
 * <p>Running: following jar is required
 * <ul><li>jfor.jar</li></ul></p>
 *
 *@author    triolet
 */
public final class RTFSerializer extends TSerializer {
	/** the writer*/
	private Writer rtfWriter = null;
	/** a handler to convert fo to rtf*/
	private Converter handler = null;

	/** format of the file  */
	String format;

	/**
	 * Constructor for the RTFSerializer object
	 *
	 *@param  pIn                in ports
	 *@param  pParam             the parameters of this serializer
	 *@param  pStaticAttributes  Description of the Parameter
	 *@exception  SAXException   Any SAX exception, possibly wrapping another exception.
	 *@exception  IOException    Description of the Exception
	 */
	public RTFSerializer(String[] pIn, Parameters pParam, StringParameters pStaticAttributes)
			 throws SAXException, IOException {
		super(pIn, pParam, pStaticAttributes);
		if (pParam.getParameter("format") != null) {
			format = (String) pParam.getParameter("format");
		} else {
			format = "rtf";
		}
	}

	/**
	 *  Gets the contentHandler attribute of the PDFSerializer object
	 *
	 *@return    The contentHandler value
	 */
	public ContentHandler getContentHandler() {
		return handler;
	}

	/**  
	 * Sets the outputStream attribute of the RTFSerializer object 
	 */
	public void setOutputStream() {
		super.setOutputStream();
		if (fileName.lastIndexOf('$') == -1) { //all parameters have been bound
			if (!format.equals("")) {
				fileName = fileName.substring(0, fileName.lastIndexOf('.') + 1) + format;
			}
			try {
				FileOutputStream FOS = null;
				FOS = new FileOutputStream(new File(fileName));
				rtfWriter = new BufferedWriter(new OutputStreamWriter(FOS));
				handler = new Converter(rtfWriter, Converter.createConverterOption());
				//generatePort();
			} catch (IOException IOE) {
				System.err.println("[RTFSerializer]error : " + IOE);
				System.err.println("[RTFSerializer]error : f=" + fileName);
				IOE.printStackTrace();
			} //end try
			catch (Exception e) {
				System.err.println("[RTFSerializer]error : " + e);
				e.printStackTrace();
			}//end try
		}
	}

	/**Generates ports for this component */
	public void generatePort() {
		super.generatePort((ContentHandler) handler);
	}
}

