/**
 * $Id: Dispatch.java,v 1.15 2002/07/05 10:30:04 gchomat Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
  * Transmorpher graph call node interface
  *
  * @author Jerome.Euzenat@inrialpes.fr
  * @since jdk 1.3 / SAX 2.0
  */

package fr.fluxmedia.transmorpher.Graph ;

import fr.fluxmedia.transmorpher.Files.ReadWrite;
import fr.fluxmedia.transmorpher.TProcessFactory.TProcessFactory;
import fr.fluxmedia.transmorpher.TProcessFactory.TProcessComposite;
import fr.fluxmedia.transmorpher.TProcessFactory.TProcess;

public class Dispatch extends CallImpl {

    public Dispatch(){
	this(null,null);
    }
    
    public Dispatch(String n, String t){
	this(n,t,null);
    }
    
    public Dispatch(String n, String t, Process p){
	this(n,t,p,0);
    }
    
    public Dispatch (String n, String t, Process p, int out){
	super(n, t, p, 1/*Nb In*/, out);
    }
    
    public PortList outPorts(){
	return outPorts;
    }



  /** Propagates the nullity of all its output to its sole input */
  public void retroNull( PortList out, PortList in) {
      nullified = true;
      for ( int i=1; nullified && i != out.length() ; i++){
	  if ( !(out.getPort(i).getChannel().nullifiedP()) )
	      nullified = false;
      } // end for
      in.getPort(0).getChannel().nullify();
      if ( nullified ) in.getPort(0).getChannel().nullify();
      visited = true;
      
      //notify observer
      update(null);
  }
  
  public void generateXML (){
    System.out.print("      <dispatch id=\""+id+"\" type=\""+getType()+"\" ");
    System.out.print("in=\"");
    inPorts.generateXML();
    System.out.print("\" out=\"");
    outPorts.generateXML();
    System.out.println("\"/>");
  }
  
 public  final void createProcess(TProcessComposite currentProcess, TProcessFactory iProcessFactory) {                                
   String[] vPortIn = inPorts().toStringList();
   String[] vPortOut = outPorts().toStringList();

   TProcess vCurrent = (TProcess)TProcessFactory.newDispatcher(vPortIn,vPortOut,
						     getAttributes().getStringParameter("type"),
						     getParameters());
   currentProcess.addProcess(vCurrent);
   update(vCurrent);
  } //end proc
 
  public void generateJavaCode(ReadWrite file){
    if(file == null) {
      System.out.println(getId()+" [generateCode]: the file is null!!");
    } else {
      file.Writeln();
      file.Writeln(4,"//------------------ Generation of Dispatcher "+getId()+" -------------------");
      generatePorts(file);
      generateParameters(file);
      file.Writeln(4,"tmCurrentProcess = tmProcessFactory.newDispatcher(portIn,portOut,\""+getType()+"\",tmParameters);");
      file.Writeln(4,"tmCurrentProcess.setName(\""+getId()+"\");");
      file.Writeln();
      file.Writeln(4,"tmCurrentCompositeProcess.addProcess(tmCurrentProcess);");
    } //end if
  } //end proc
} //end class
