/**
 * $Id: MainProcess.java,v 1.6 2002/10/02 06:04:00 triolet Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
  * Transmorpher graph processes definitions
  *
  * @author Jerome.Euzenat@inrialpes.fr
  * @since jdk 1.3 / SAX 2.0
  */

package fr.fluxmedia.transmorpher.Graph ;

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.ListIterator;
import java.util.Stack;
import java.util.Date;

import fr.fluxmedia.transmorpher.Utils.LinearIndexedStruct;
import fr.fluxmedia.transmorpher.Utils.Version;
import fr.fluxmedia.transmorpher.Files.ReadWrite;
import fr.fluxmedia.transmorpher.TProcessFactory.TProcessFactory;
import fr.fluxmedia.transmorpher.TProcessFactory.TProcessComposite;
import fr.fluxmedia.transmorpher.TProcessFactory.TApplyProcess;
import fr.fluxmedia.transmorpher.TProcessFactory.TProcess;
import fr.fluxmedia.transmorpher.TProcessFactory.TMain;

public abstract class MainProcess extends fr.fluxmedia.transmorpher.Graph.Process {

    public MainProcess(String n, Transmorpher t, int nin, int nout){
	super(n,t,nin,nout);
    }
    
    public abstract TProcessComposite createMainProcess(TProcessFactory iProcessFactory);
    
    public void generateJavaCode(ReadWrite iFile,LinearIndexedStruct externs) {
	iFile.Writeln("/** Generated by "+Version.NAME+" "+Version.RELEASE);
	iFile.Writeln("    See "+Version.URL);
	Date date = new Date();
	iFile.Writeln("    "+date+" **/");
	iFile.Writeln();
	iFile.Writeln("// import Java classes");
	iFile.Writeln("import java.util.Stack;");
	iFile.Writeln();
	generateImport(iFile,externs);
	iFile.Writeln("// import transmorpher classes");
	iFile.Writeln("import fr.fluxmedia.transmorpher.TProcessFactory.*;");
	iFile.Writeln("import fr.fluxmedia.transmorpher.Utils.*;");
	iFile.Writeln();
	generateClassHeader(iFile);
	iFile.Writeln(4," // ressource manager");
	iFile.Writeln(4,"static SystemResources vSR = null;");
	iFile.Writeln();
	iFile.Writeln(4,"//global variables for port");
	iFile.Writeln(4,"static String[] portIn  = null ;");
	iFile.Writeln(4,"static String[] portOut = null;");
	iFile.Writeln(4,"static String[] LocIn  = null ;");
	iFile.Writeln(4,"static String[] LocOut = null;");
	iFile.Writeln();
	iFile.Writeln(4," // global variable for the processFactory");
	// BUG: This is not really compiling since stuff are created at runtime!
	iFile.Writeln(4,"static TProcessFactory tmProcessFactory = null;");
	iFile.Writeln();
	iFile.Writeln(4,"private static LinearIndexedStruct newDefexterns(){");
	iFile.Writeln(6,"LinearIndexedStruct defs = new LinearIndexedStruct();");
	Enumeration keys = externs.getKeys();
	while(keys.hasMoreElements()){
		String name = (String) keys.nextElement();
	    iFile.Writeln(6,"defs.add(\""+name+"\", (Object)\""+(String)externs.get(name)+"\");");
	};
	iFile.Writeln(6,"return defs;");
	iFile.Writeln(4,"}");
	iFile.Writeln(4," // stack of composite processes");
	iFile.Writeln(4,"static Stack tmProcessStack = null;");
	iFile.Writeln();
	iFile.Writeln(4," // global variables for attributes and parameters");
	// Default values should be that of Transmorpher
	iFile.Writeln(4,"static int debug_mode = 0;");
	iFile.Writeln(4,"static Parameters tmParameters = null ;");
	iFile.Writeln(4,"static StringParameters tmAttributes = null;");
	iFile.Writeln();
	generateMainHeader(iFile);
	//temporaire
	iFile.Writeln(4,"double begin = System.currentTimeMillis();");
	iFile.Writeln(4,"double end = 0;");
	iFile.Writeln(4,"TProcessComposite tmCurrentCompositeProcess = null;");
	iFile.Writeln(4,"TProcess tmCurrentProcess = null;");
	generateReadParameters(iFile);
	generateBody(iFile,externs);
	generateEnd(iFile);
    }
    
    public abstract void generateImport(ReadWrite iFile,LinearIndexedStruct externs);
    public abstract void generateClassHeader(ReadWrite iFile);
    public abstract void generateMainHeader(ReadWrite iFile);
    public abstract void generateReadParameters(ReadWrite iFile);
    public abstract void generateBody(ReadWrite iFile,LinearIndexedStruct externs);
    public abstract void generateEnd(ReadWrite iFile);

} //end class
