/**
 * $Id: ProcessParser.java,v 1.57 2002/10/16 08:03:25 triolet Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001-2002 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package fr.fluxmedia.transmorpher.Parser ;

// Imported XML classes

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerConfigurationException;

//Imported SAX classes

import org.xml.sax.SAXException;
import org.xml.sax.Attributes;

//Imported JAVA classes

import java.io.*;
import java.io.IOException;

import java.util.Observer;
import java.util.Stack;
import java.util.Vector;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.Observer;
import java.lang.Integer;

//Imported Transmorpher classes

import fr.fluxmedia.transmorpher.Utils.Parameters ;
import fr.fluxmedia.transmorpher.Files.* ;
import fr.fluxmedia.transmorpher.Graph.* ;
import fr.fluxmedia.transmorpher.Graph.Rules.* ;

public class ProcessParser extends FMParser {
    protected String uri = null;
    protected String tmp = "";
    protected Transmorpher currentTransmorpher = null;
    protected Transformation currentTransformation = null;
    protected Call currentCall = null;
    protected ModTag currentRule = null;
    protected Rule rule = null;
    protected String currentParam = null;
    protected String currentWParam = null;
    protected Iterator currentIterator = null;
    protected Parameters currentParams = null;
    
    protected Object[] observers;
    
    RuleSetParser myRuleSetParser = null ;
    QueryParser myQueryParser = null ;
    
    /** Are we copying some XSLT */
    boolean  inLitteralXSLT = false;
    
    boolean inRuleSet = false ;
    boolean inQuery = false ;
    
  /*-------------------------------------------- CONSTRUCTOR  -----------------------------*/
    
  /** The constructor, build the XML PArser but not parse the document */
    public ProcessParser( int debug_mode) {
	this(debug_mode,null);
    }
    
    public ProcessParser(int debug_mode, Object[] observers){
	super(debug_mode);
	myRuleSetParser = new RuleSetParser(debug_mode,this);
	myQueryParser = new QueryParser(debug_mode,this);
	this.observers = observers;
    }

    
  /*--------------------------------------------- END CONSTRUCTOR -------------------------*/

  /** Parse the document given in parameter: Same this as FMParser but with Transmorpher */
  public Transmorpher Newparse(String u) {
    uri = u;
    try {
      iParser.parse(uri,this);
    } catch (SAXException SE) {
      System.err.println("[FMParser]Erreur SAX(" + SE+")");
      System.err.println("[FMParser]Erreur SAX(" + u+")");
      System.err.println("[FMParser]Erreur SAX(" + SE.getMessage()+")");
      System.err.println("[FMParser]Erreur SAX(" + SE.getException()+")");
      SE.printStackTrace();
    } catch (java.io.IOException IOE) {
      System.err.println("[FMParser]Erreur IO(" + IOE+")");
    }
    return currentTransmorpher;
  }
    
    /* ************* */
    /* START ELEMENT */
    /* ************* */
    
  /** Call by the XML parser at the begining of an element */
    public  void startElement(String namespaceURI, String localName, String qname, Attributes atts) {
	if(debugMode > 2) 
	    System.err.println("startElement ProcessParser : " + localName);
	if (inRuleSet) {
	    myRuleSetParser.startElement(namespaceURI,localName,qname,atts);
	} else if (inQuery) {
	    myQueryParser.startElement(namespaceURI,localName,qname,atts);
	} else if ( inLitteralXSLT ){
	    //JE: here we should not do that but put the XSLT in a XSLTTemplate element
	    //if(atts!=null) System.out.println(atts.getValue("match"));
	    //iXslFile.Writeln(2,"<"+qname+" "+listAttribute(atts)+">");
	} else if (namespaceURI.equals("http://transmorpher.fluxmedia.fr/1.0")) {
	    String[] inChannels = AnalysePortNames(atts.getValue("in"));
	    String[] outChannels = AnalysePortNames(atts.getValue("out"));
	    if(debugMode > 2) System.err.println("startElement ProcessParser : " + localName);
	    if (localName.equals("with-param")) {
		currentWParam = atts.getValue("name");
		if ( atts.getValue("select") != null ) {
		    if ( currentParams != null )
			currentParams.setParameter(currentWParam,atts.getValue("select"));
		}
	    }  else if (localName.equals("param")) {
          currentParam = atts.getValue("name");
          if ( atts.getValue("select") != null ) {
	      currentParams.setParameter(currentParam,atts.getValue("select"));
          }
	/*/*************** GENERATE **************/
	    } else if (localName.equals("generate")) {
          currentCall = (Call)new Generate(atts.getValue("id"),atts.getValue("type"),
                 (fr.fluxmedia.transmorpher.Graph.Process)currentTransformation,
                 outChannels.length,
                  // Compatibility
                  atts.getValue("file"));

           fixOutPorts( currentCall, currentCall.outPorts(), outChannels, 1, "generate");
	   currentParams = currentCall.getParameters();
	   ((fr.fluxmedia.transmorpher.Graph.Process)currentTransformation).addCall((Call)currentCall);
	/*/*************** SERIALIZE **************/
	} else if (localName.equals("serialize")) {
          currentCall = (Call)new Serialize(atts.getValue("id"),atts.getValue("type"),
                                         (fr.fluxmedia.transmorpher.Graph.Process)currentTransformation,
                                          inChannels.length,
                                          // Compatibility
                                          atts.getValue("file"));

	  fixInPorts( currentCall, currentCall.inPorts(), inChannels, 1, "serialize");
	  currentParams = currentCall.getParameters();
	  ((fr.fluxmedia.transmorpher.Graph.Process)currentTransformation).addCall((Call)currentCall);
	/*/*************** DISPATCH **************/
	} else if (localName.equals("dispatch")) {
          currentCall = (Call)new Dispatch(atts.getValue("id"),atts.getValue("type"),
                 (fr.fluxmedia.transmorpher.Graph.Process)currentTransformation,
                  outChannels.length);
          fixInPorts( currentCall, currentCall.inPorts(), inChannels, 1, "dispatch");
          fixOutPorts( currentCall, currentCall.outPorts(), outChannels, -1, "dispatch");
	  currentParams = currentCall.getParameters();
	  ((fr.fluxmedia.transmorpher.Graph.Process)currentTransformation).addCall((Call)currentCall);
	/*/*************** MERGE **************/
        } else if (localName.equals("merge")) {
          currentCall = (Call)new Merge(atts.getValue("id"),atts.getValue("type"),
                  (fr.fluxmedia.transmorpher.Graph.Process)currentTransformation,
                  inChannels.length);
          fixInPorts( currentCall, currentCall.inPorts(), inChannels, -1, "merge");
          fixOutPorts( currentCall, currentCall.outPorts(), outChannels, 1, "merge");
	  currentParams = currentCall.getParameters();
	  ((fr.fluxmedia.transmorpher.Graph.Process)currentTransformation).addCall((Call)currentCall);
	/*/*************** APPLY-EXTERNAL **************/
	} else if (localName.equals("apply-external")) {
          currentCall = (Call)new ApplyExternal(atts.getValue("id"),atts.getValue("type"),
                                              atts.getValue("file"),
                                             (fr.fluxmedia.transmorpher.Graph.Process)currentTransformation,
                                              inChannels.length,
                                              outChannels.length);
          fixInPorts( currentCall, currentCall.inPorts(), inChannels, -1, "apply-external");
          fixOutPorts( currentCall, currentCall.outPorts(), outChannels, -1, "apply-external");
	  currentParams = currentCall.getParameters();
	  ((fr.fluxmedia.transmorpher.Graph.Process)currentTransformation).addCall((Call)currentCall);
	/*/*************** APPLY-PROCESS **************/
	} else if (localName.equals("apply-process")) {
          currentCall = (Call)new ApplyProcess(atts.getValue("id"),atts.getValue("type"),
                 (fr.fluxmedia.transmorpher.Graph.Process)currentTransformation,
                 atts.getValue("ref"),
                  inChannels.length,
                  outChannels.length);
          fixInPorts( currentCall, currentCall.inPorts(), inChannels, -1, "apply-process");
          fixOutPorts( currentCall, currentCall.outPorts(), outChannels, -1, "apply-process");
	  currentParams = currentCall.getParameters();
	  ((fr.fluxmedia.transmorpher.Graph.Process)currentTransformation).addCall((Call)currentCall);
	/*/*************** APPLY-QUERY **************/
	} else if (localName.equals("apply-query")) {
          currentCall = (Call)new ApplyQuery(atts.getValue("id"),atts.getValue("type"),
                 (fr.fluxmedia.transmorpher.Graph.Process)currentTransformation,
                 atts.getValue("ref"));
          fixInPorts( currentCall, currentCall.inPorts(), inChannels, 1, "apply-query");
          fixOutPorts( currentCall, currentCall.outPorts(), outChannels, 1, "apply-query");
      	  currentParams = currentCall.getParameters();
	  ((fr.fluxmedia.transmorpher.Graph.Process)currentTransformation).addCall((Call)currentCall);
	/*/*************** APPLY-RULESET **************/
	} else if (localName.equals("apply-ruleset")) {
          currentCall = (Call)new ApplyRuleset(atts.getValue("id"),"Ruleset",
					       (fr.fluxmedia.transmorpher.Graph.Process)currentTransformation, atts.getValue("ref"));
	  fixInPorts( currentCall, currentCall.inPorts(), inChannels, 1, "apply-ruleset");
	  fixOutPorts( currentCall, currentCall.outPorts(), outChannels, 1, "apply-ruleset");
	  ((fr.fluxmedia.transmorpher.Graph.Process)currentTransformation).addCall((Call)currentCall);
	  currentParams = currentCall.getParameters();
	/*/*************** REPEAT *********************/
        } else if (localName.equals("repeat")) {
	    String[] testChannels = AnalysePortNames(atts.getValue("test"));
	    String[] bufChannels = AnalysePortNames(atts.getValue("bin"));
	    int nbtests;
	    if ( testChannels == null ) nbtests = 0;
	    else nbtests = testChannels.length;
	    // Repeat is both Call and Process
	    // First, Call
	    currentCall = (Call)new Repeat(atts.getValue("id"),nbtests,
					   (fr.fluxmedia.transmorpher.Graph.Process)currentTransformation,
					   inChannels.length,
            				   outChannels.length,
					   bufChannels.length);
	  fixInPorts( currentCall, currentCall.inPorts(), inChannels, -1, "repeat");
	  fixOutPorts( currentCall, currentCall.outPorts(), outChannels, -1, "repeat");
          ((fr.fluxmedia.transmorpher.Graph.Process)currentTransformation).addCall((Call)currentCall);
	  // then Process
          currentTransformation = ((Repeat)currentCall).getProcessBody();
	  fixInPorts( currentTransformation, currentTransformation.inPorts(), inChannels);
	  fixBuf( currentTransformation, ((Repeat)currentCall).inBuf(),currentTransformation.inPorts(), bufChannels, inChannels.length);
          fixOutPorts( currentTransformation, currentTransformation.outPorts(), outChannels);
	  
	  currentParams = currentCall.getParameters();
	/*/*************** ITERATE *******************/
        } else if (localName.equals("iterate")) {
	    currentIterator = (Iterator)new Iterator(atts.getValue("name"),atts.getValue("type"),
						     (Repeat)currentCall);
	    ((Repeat)currentCall).addIterator(currentIterator);
	    currentParams = currentIterator.getParameters();
	/*/*************** PROCESS *******************/
        } else if (localName.equals("process")) {
	    currentTransformation = new fr.fluxmedia.transmorpher.Graph.Process(atts.getValue("name"),currentTransmorpher,inChannels.length,outChannels.length);
	    fixInPorts( currentTransformation, currentTransformation.inPorts(), inChannels);
	    fixOutPorts( currentTransformation, currentTransformation.outPorts(), outChannels);
	    currentTransmorpher.addTransformation(atts.getValue("name"),currentTransformation);
	    currentParams = currentTransformation.getParameters();
	/*/*************** QUERY *********************/
        } else if (localName.equals("query")) {
	    inQuery = true ;
	    myQueryParser.setTransmorpher (currentTransmorpher);
	    myQueryParser.startElement(namespaceURI,localName,qname,atts);
	/*/*************** RULESET *******************/
        } else if (localName.equals("ruleset")) {
	    inRuleSet = true ;
          myRuleSetParser.setTransmorpher (currentTransmorpher);
          myRuleSetParser.startElement(namespaceURI,localName,qname,atts);
	/*/*************** TRANSMORPHER **************/
        } else if (localName.equals("transmorpher")) {
          currentTransmorpher = new Transmorpher(
						 atts.getValue("name"),
						 atts.getValue("version"),
						 debugMode,
						 ( (atts.getValue("optimized") != null) && !(atts.getValue("optimized").equals("true"))) );
          currentTransmorpher.setFile( new File(uri) );
	  
	  // add observer to transmorpher
	  if(observers != null)
	      {
		  for(int i = 0 ; i<observers.length ; i++){
		      if(observers[i] instanceof Observer)
			  currentTransmorpher.addObserver((Observer)observers[i]);
		  }
		}
	/*/*************** DEFEXTERN *****************/
	} else if (localName.equals("defextern")) {
	    currentTransmorpher.addExtern(atts.getValue("name"),atts.getValue("class"),atts.getValue("implements"),
					( (atts.getValue("default") != null) && !(atts.getValue("default").equals("true"))));
	/*/*************** INCLUDE *****************/
        } else if (localName.equals("include")) {
	    currentTransmorpher.addInclude(atts.getValue("href"));
	/*/*************** IMPORT **************/
        } else if (localName.equals("import")) {
	    currentTransmorpher.addImport(atts.getValue("href"));
	/*/*************** MAIN **************/
        } else if (localName.equals("main")) {
	    currentTransformation = (fr.fluxmedia.transmorpher.Graph.Process)new Main(atts.getValue("name"),currentTransmorpher);
	    if ( currentTransmorpher.getMain() == null) {
		currentTransmorpher.setMain((MainProcess)currentTransformation);
	    } else {
            System.err.println("Error: not two main in transmorpher stylesheet");
	    }
	    currentTransmorpher.addTransformation(atts.getValue("name"),(Transformation)currentTransformation);
	    currentParams = currentTransformation.getParameters();
	/*/*************** SERVLET **************/
        } else if (localName.equals("servlet")) {
	    currentTransformation = (fr.fluxmedia.transmorpher.Graph.Process)new Servlet(atts.getValue("name"),currentTransmorpher);
	    if ( currentTransmorpher.getMain() == null) {
		currentTransmorpher.setMain((MainProcess)currentTransformation);
	    } else {
		System.err.println("Error: not two main in transmorpher stylesheet");
	    }
	    currentTransmorpher.addTransformation(atts.getValue("name"),(Transformation)currentTransformation);
	    currentParams = currentTransformation.getParameters();
	/*/*************** TRANSFORMER **************/
        } else if (localName.equals("transformer")) {
	    currentTransformation = (fr.fluxmedia.transmorpher.Graph.Process)new Transformer(atts.getValue("name"),currentTransmorpher);
	    if ( currentTransmorpher.getMain() == null) {
		currentTransmorpher.setMain((MainProcess)currentTransformation);
	    } else {
		System.err.println("Error: not two main in transmorpher stylesheet");
          }
          currentTransmorpher.addTransformation(atts.getValue("name"),(Transformation)currentTransformation);
	        currentParams = currentTransformation.getParameters();
        } else {
          System.err.println("[ProcessParser]Error: Unknown element : " + localName);
      } //end if
    } else if (namespaceURI.equals("http://www.w3.org/1999/XSL/Transform") && localName.equals("template")) {
      rule = (Rule)new XSLTTemplate(atts.getValue("match"),atts.getValue("mode"));
      ((Ruleset)currentTransformation).addRule( (Rule)rule );
      inLitteralXSLT = true;
      // JE: Copy the structure or the sax events?
      // JE: Possibility to write in a string?
      //iXslFile.Writeln(2,"<"+qname+" "+listAttribute(atts)+" >");
    } else {
      System.err.println("Error : Unknown namespace : " + namespaceURI);
      System.err.println("fullname: " + qname );
      System.err.println("for element :" +localName);
      System.err.println();
    }
  } // startElement(String namespaceURI, String localName, String qname, Attributes atts)
  

  /* The content of the tags */
  /* No content in Transmorpher so far but in XSLT code */
  
    public void characters(char ch[], int start, int length) {
  
	String content = tmp+"";
	int end = start + length;
	for( int i = start; i < end ; i++ ) content = content + ch[i];
	if(debugMode > 2) System.err.println("Content : " + content);
	if( inLitteralXSLT == true ) {
	    // JE: to be put in the corresponding string
	    //iXslFile.Writeln(2,content);
	   
	} 
	tmp=content;
	/* else if (end==2048){	
			// FT : ch[] can only have 2048 characters. (xerces implementation)
			tmp=content;
		}
		else{
			if( currentParam != null ) {
				currentParams.setParameter(currentParam,content);
				tmp="";
			} else if ( currentWParam != null ) {
				currentParams.setParameter(currentWParam,content);
				tmp="";
			}
		} */
    }


  /* *********** */
  /* END ELEMENT */
  /* *********** */

  public void setEndRuleSet() {
     inRuleSet =false ; 
  }
  public void setEndQuery() {
     inQuery =false ; 
  }

  /** Call by the XML parser at the end of an element */
  public  void endElement(String namespaceURI,String pName, String qName ) {
    if(debugMode > 2) System.err.println("endElement ProcessParser : " + pName);
  
  if (inRuleSet) {
      myRuleSetParser.endElement(namespaceURI,pName,qName);
    } else if (inQuery) {
      myQueryParser.endElement(namespaceURI,pName,qName);
    } else if( inLitteralXSLT ){
      //iXslFile.Writeln(2,"</"+qName+">");
      if( namespaceURI.equals("http://www.w3.org/1999/XSL/Transform") && pName.equals("template") )
        inLitteralXSLT = false;
  } else if(namespaceURI.equals("http://transmorpher.fluxmedia.fr/1.0"))  {
      if (pName.equals("param")) {
	      if( currentParam != null ) {
		      currentParams.setParameter(currentParam,tmp);
			tmp="";
		} else if ( currentWParam != null ) {
			currentParams.setParameter(currentWParam,tmp);
			tmp="";
		}
	  currentParam = null;
      } else if (pName.equals("with-param")) {
	      if( currentParam != null ) {
		      currentParams.setParameter(currentParam,tmp);
			tmp="";
		} else if ( currentWParam != null ) {
			currentParams.setParameter(currentWParam,tmp);
			tmp="";
		}
	  currentWParam = null;
      } else if (pName.equals("generate")) {
	  cleanStack();
      } else if (pName.equals("serialize")) {
	  cleanStack();
      } else if (pName.equals("dispatch")) {
	  cleanStack();
      } else if (pName.equals("merge")) {
	  cleanStack();
      }	else if (pName.equals("defextern")) {

      } else if (pName.equals("apply-external")) {
	  cleanStack();
      } else if (pName.equals("apply-process")) {
	  cleanStack();
      } else if (pName.equals("apply-query")) {
	  cleanStack();
      } else if (pName.equals("apply-ruleset")) {
	  cleanStack();
      } else if (pName.equals("repeat")) {
	  checkChannels( (fr.fluxmedia.transmorpher.Graph.Process)currentTransformation );
	  currentTransformation = currentCall.getProcess();
	  currentParams = currentTransformation.getParameters();
	  cleanStack();
      } else if (pName.equals("iterate")) {
	  currentCall = currentIterator.getRepeat();
	  currentParams = currentCall.getParameters();
      } else if (pName.equals("extends")) {
      } else if (pName.equals("process")) {
    checkChannels( (fr.fluxmedia.transmorpher.Graph.Process)currentTransformation );
    ((fr.fluxmedia.transmorpher.Graph.Process)currentTransformation).setUp();
    currentTransformation = null;
      } else if (pName.equals("main")) {
    checkChannels( (fr.fluxmedia.transmorpher.Graph.Process)currentTransformation );
    ((Main)currentTransformation).setUp();
    currentTransformation = null;
      } else if (pName.equals("transformer")) {
    checkChannels( (fr.fluxmedia.transmorpher.Graph.Process)currentTransformation );
    ((Transformer)currentTransformation).setUp();
    currentTransformation = null;
      } else if (pName.equals("servlet")) {
    checkChannels( (fr.fluxmedia.transmorpher.Graph.Process)currentTransformation );
    ((Servlet)currentTransformation).setUp();
    currentTransformation = null;
      } else if (pName.equals("transmorpher")) {
      } else {
	System.err.println("[ProcessParser] Error : Unknown element : " + pName);
      } //end if

  }
     else System.err.println("Error : Unknown namespace : "+namespaceURI);
  
     } //end endeLement
    
    /*--------------------- CREATE THE PORTS FROM THE ARGUMENT LIST ----------------------------------*/

    // JE: Concerning transformations, everything should be simpler because nothing
    // is created yet so we just have to register. But, OK.
    // This is only used for processes!!i
    // FT: if we are in repeat transformation, all INPUTs are FeedBacks not Channels

protected void fixInPorts( Transformation t, PortList list, String[] portNames) {
  for( int i = 0; i != portNames.length; i++) {
	list.setPort(i, new Port( portNames[i], (Transformation)t, i,Port.INPUT ));
 	Port p=list.getPort(i);
               if (t instanceof RepeatProcess){
                        String name = p.getName();
                        if ( (name.equals("_null_") || name.equals("_stdout_") || name.equals("_stdin_")) ) {
                                System.err.println("Error: cannot use "+name+" in repeat buffers");
                        } else {
                                FeedBack f=null;
                                if (((fr.fluxmedia.transmorpher.Graph.Process)currentTransformation).getChannel( name )instanceof FeedBack)
                                        f = (FeedBack)((fr.fluxmedia.transmorpher.Graph.Process)currentTransformation).getChannel( name );
                                if ( f == null ) {
                                        f = new FeedBack( name, (fr.fluxmedia.transmorpher.Graph.Process)currentTransformation );
                                        ((fr.fluxmedia.transmorpher.Graph.Process)currentTransformation).addChannel( (Channel)f );
                                }
                                f.setIn( p );
                                p.setChannel( (Channel)f );

                        }
                }
  }
  registerChannelIn( list, false );
}

protected void fixOutPorts( Transformation t, PortList list, String[] portNames) {
  for( int i = 0; i != portNames.length; i++) {
      list.setPort(i, new Port( portNames[i], (Transformation)t, i,Port.OUTPUT ));
  }
  registerChannelOut( list, false );
}

/*protected void fixTests( Repeat r, PortList list, String[] portNames) {
    if ( portNames != null ){
  for( int i = 0; i != portNames.length; i++) {
      list.setPort(i, new Port( portNames[i], r, i ));
      Port p = list.getPort(i);
      String name = p.getName();
  if ( (name.equals("_null_") || name.equals("_stdout_") || name.equals("_stdin_")) ) {
      System.err.println("Error: cannot use "+name+" in test buffers");
  } else {
    Test t = new Test( name, (fr.fluxmedia.transmorpher.Graph.Process)currentTransformation );
    ((fr.fluxmedia.transmorpher.Graph.Process)currentTransformation).addChannel( (Channel)t );
    p.setChannel( (Channel)t );
  }
  }
  }
}*/

protected void fixBuf(Transformation t , PortList bufList,PortList inList, String[] portNames, int expected) {
  if ( portNames.length != expected ) {
      System.err.println("Error: Repeat must have as many ("+expected+") buffers as input (given "+portNames.length+")");
  } else {
      
    for( int i = 0; i != portNames.length; i++) {
      bufList.setPort(i, new Port( portNames[i],t , i));
      Port p = bufList.getPort(i);
      Port inP = inList.getPort(i);

      String name = p.getName();
      String inName= inP.getName();
      if ( (name.equals("_null_") || name.equals("_stdout_") || name.equals("_stdin_")) ) {
      		System.err.println("Error: cannot use "+name+" in repeat buffers");
  	} else {
		FeedBack buf=null;
		FeedBack in=null;
		if (((fr.fluxmedia.transmorpher.Graph.Process)currentTransformation).getChannel( name )instanceof FeedBack){
     			buf = (FeedBack)((fr.fluxmedia.transmorpher.Graph.Process)currentTransformation).getChannel( name );
			buf.setOut(p); //innerOUT
		}
		else{
     			buf = new FeedBack( name, (fr.fluxmedia.transmorpher.Graph.Process)currentTransformation );
    			((fr.fluxmedia.transmorpher.Graph.Process)currentTransformation).addChannel( (Channel)buf );
			buf.setOut(p); //set innerOutput
      		}
    		buf.setIn( p );
    		p.setChannel( (Channel)buf );
     		in  = (FeedBack)((fr.fluxmedia.transmorpher.Graph.Process)currentTransformation).getChannel( inName );
		in.setIn(p);
  	}
    }
  }
}
    

protected void fixOutBuf( Repeat r, PortList list, String[] portNames, int expected) {
  if ( portNames.length != expected ) {
      System.err.println("Error: Repeat must have as many ("+expected+") buffers as output (given "+portNames.length+")");
  } else {
  for( int i = 0; i != portNames.length; i++) {
      list.setPort(i, new Port( portNames[i], r, i ));
  }
  registerChannelOut( list, false );
  }  
}

protected void fixInPorts( Call c, PortList list, String[] portNames, int expected, String type) {
  if ( (expected != -1) && (portNames.length != expected) ) {
      System.err.println("Error: "+type+" must have "+expected+" input (given "+portNames.length+")");
  } else {  	
	for( int i = 0; i != portNames.length; i++) {
		list.setPort(i, new Port( portNames[i], c, i ,Port.INPUT));
		Port p=list.getPort(i);
	}
	registerChannelOut( list, true );
	
  }
}

protected void fixOutPorts( Call c, PortList list, String[] portNames, int expected, String type) {
  if ( (expected != -1) && (portNames.length != expected) ) {
      System.err.println("Error: "+type+" must have "+expected+" input (given "+portNames.length+")");
  } else {
      for( int i = 0; i != portNames.length; i++) {
        list.setPort(i, new Port( portNames[i], c, i , Port.OUTPUT));
      }
  }
  registerChannelIn( list, true );
}

public void checkChannels( fr.fluxmedia.transmorpher.Graph.Process t ) {
  if(debugMode>1){  
 	 System.err.println("************** CHECK "+t.getName()+" ***************");
  }
  for( Enumeration e = t.getChannels(); e.hasMoreElements();) {
  // JE: Using instanceof should be less stupid...
  Channel c = (Channel)e.nextElement();
  if(debugMode>1){
 	 if (c instanceof FeedBack){
		 System.err.println("FEEDBACK "+c+" INPUT "+(c.in()).getName()+" OUTPUT "+(c.out()).getName());
	 	 if (( ((FeedBack) c).innerInput())!=null)	
			 System.err.println("INNER INPUT "+( ((FeedBack)c).innerInput()).getName());	
	 	 if (( ((FeedBack) c).innerOutput())!=null)	
			 System.err.println("INNER OUTPUT "+( ((FeedBack)c).innerOutput()).getName());	
 	}  else
        	 System.err.println("CHANNEL "+c+" INPUT "+(c.in()).getName()+" OUTPUT "+(c.out()).getName());
  }
  // System.err.println(c.getName()+" : "+c.in()+" --> "+c.out());
  if ( c instanceof Null ) {
      // No cause for problem... so far
  } else if ( c instanceof StdIn ) {
      // No cause for problem... so far
  } else if ( c instanceof StdOut ) {
      // No cause for problem... so far
  } else if ( (c.in() == null) || (c.out() == null) ) {
      System.err.println("Error: dangling channel ("+c.getName()+") in "+t.getName());
  }
    }
}

/** The algorithm depends on the context (transformation or call)
 * If we are in a Transformation:
 * the in are input for the channel, they cannot be anything but Channel
 * the out are output for the channel, they cannot be anything but Channel
 * If we are in a Call:
 * the in are output to the channel, they cannot be anything but Channel and _stdin_
 * the out are input to the channel, they cannot be anything but Channel and _stdout_ or _null_
 **/

    /** Register a channel with its input */
public void registerChannelIn( PortList list, boolean Callp ) {
    for( int i = list.length()-1; i != -1; i-- ) {
	Port p = list.getPort(i);
	String name = p.getName();
	Channel c;
	if ( !Callp && (name.equals("_null_") || name.equals("_stdout_")) ) {
	    System.err.println("Error: cannot use "+name+" in transformation signature");
	} else if ( name.equals("_stdin_") ) {
	    System.err.println("Error: cannot use "+name+" as output channel");
	} else if ( name.equals("_stdout_") ) {
	    // JE: also check if current transformation is main
	    if ( ((fr.fluxmedia.transmorpher.Graph.Process)currentTransformation).getChannel( name ) != null ){
		System.err.println("Error: Two process writing to _stdout_");
	    } else {
		c = new StdOut( name, (fr.fluxmedia.transmorpher.Graph.Process)currentTransformation );
		((fr.fluxmedia.transmorpher.Graph.Process)currentTransformation).addChannel( c );
		c.setIn( p );
		p.setChannel( c );
	    }
	} else if ( name.equals("_null_") ) {
	    c = new Null( name, (fr.fluxmedia.transmorpher.Graph.Process)currentTransformation );
	    ((fr.fluxmedia.transmorpher.Graph.Process)currentTransformation).addChannel( c );
	    c.setIn( p );
	    // TAKE CARE there could be several (but useless)
	    p.setChannel( c );
	} else {
	    c = ((fr.fluxmedia.transmorpher.Graph.Process)currentTransformation).getChannel( name );
	    if ( c != null ) {
		if ( (c.in() == null) || (c instanceof FeedBack) ) {// && take care the the innerInput is not already set...
		    c.setIn(p);
		    p.setChannel( c );
		} else {
		    System.err.println("Error: two in pours on the same channel"+c.getName()+" "+p.getName());
		}
	    } else {
		c = new Channel( name, (fr.fluxmedia.transmorpher.Graph.Process)currentTransformation );
		((fr.fluxmedia. transmorpher.Graph.Process)currentTransformation).addChannel( c );
		c.setIn( p );
		p.setChannel( c );
	    }
	}
    }
}

    /** Register a channel with its output */
public void registerChannelOut( PortList list, boolean Callp ) {
    for( int i = list.length()-1; i != -1; i-- ) {
	Port p = list.getPort(i);
	String name = p.getName();
	Channel c;
	if ( name.equals("_null_") || name.equals("_stdout_") ) {
	    System.err.println("Error: cannot use "+name+" as output channel");
	} else if ( !Callp && name.equals("_stdin_") ){
	    System.err.println("Error: cannot use "+name+" in transformation signature");
	} else if ( name.equals("_stdin_") ){
	    // JE: also check if current transformation is main
	    if ( ((fr.fluxmedia.transmorpher.Graph.Process)currentTransformation).getChannel( name ) != null ){
		System.err.println("Error: Two process listening to _stdin_");
	    } else {
		c = new StdIn( name, (fr.fluxmedia.transmorpher.Graph.Process)currentTransformation );
		((fr.fluxmedia.transmorpher.Graph.Process)currentTransformation).addChannel( c );
		c.setOut( p );
		// TAKE CARE there could be several (but useless)
		p.setChannel( c );
	    }
	} else {
	    c = ((fr.fluxmedia.transmorpher.Graph.Process)currentTransformation).getChannel( name );
	    if ( c != null ) {
		
		if ( (c.out() == null) || (c instanceof FeedBack) ) {// && take care the the innerOutput is not already set...
		    if ((c instanceof FeedBack)&&(((FeedBack) c).innerOutput()==null)){
			c.setOut( p ); //set innerOutput
			if (c.out()==null)
			c.setOut( p) ; //set output
		    }
		    else
		    	c.setOut(p);
		    p.setChannel( c );
		    
		((fr.fluxmedia.transmorpher.Graph.Process)currentTransformation).addChannel( c );
		} else {
		    System.err.println("Error: two out pours on the same channel "+c.getName()+" "+p.getName());
		}
	    } else {
		
		c = new Channel( name, (fr.fluxmedia.transmorpher.Graph.Process)currentTransformation );
		((fr.fluxmedia.transmorpher.Graph.Process)currentTransformation).addChannel( c );
		c.setOut( p );
	    }
	}
    }
}


    /*---------------------------- ANALYSE PORT NAME -----------------------------------------*/
    
  /** Analyze the port Name attributes */
  public static  String[] AnalysePortNames(String pListe) {
      if ( pListe != null ) {
  // JE: I would like to add "," in the tokenizer (possible?).
    StringTokenizer vTokens = new StringTokenizer( pListe, " ", true );
    String[] vAns = new String[vTokens.countTokens()/2+1];
    String vToken = null;
    int i = 0;
    while ( vTokens.hasMoreTokens() ) {
      vToken = vTokens.nextToken();
      if (vToken.equals(" ")) {
      } else {
        vAns[i++]=vToken;
      } //end if
    }//end while
    return vAns;
      } else {
	  return null;
      }
  } //end proc
    
    /** Reset the parameters as they were after taking care of a Call */
    protected void cleanStack(){
	currentParams = currentCall.getProcess().getParameters();
	if ( (currentTransformation != null) && (currentTransformation instanceof RepeatProcess) ){
	    currentCall = ((RepeatProcess)currentTransformation).getCaller();
	} else {
	    currentCall = null;
	}
    }
    
    /*------------------------------------ LIST ATTRIBUTE -----------------------------------*/

    /** Returns a XML-attribute string from the result of such a string */
    // Used only for transmitting litteral XSLT Templates
    public String listAttribute(Attributes atts){
  String listAttribute="";
  if (atts.getLength()!=0)
      {
    for(int i=0 ; i<atts.getLength() ; i++){
        listAttribute=listAttribute+atts.getLocalName(i)+"=\""+atts.getValue(i)+"\" ";
    }
    
      }
  
      return listAttribute;
    }
    
}//end class
    
