/**
 *  $Id: TLoop.java,v 1.17 2002/09/13 05:54:10 triolet Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001-2002 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package fr.fluxmedia.transmorpher.TProcessFactory;

// Imported fr.fluxmedia.transmorpher classes
import fr.fluxmedia.transmorpher.Files.ReadWrite;
import fr.fluxmedia.transmorpher.TProcessFactory.LoopHandler;
import fr.fluxmedia.transmorpher.TProcessFactory.TIterator;
import fr.fluxmedia.transmorpher.Utils.Parameters;
import fr.fluxmedia.transmorpher.Utils.StringParameters;
import fr.fluxmedia.transmorpher.stdlib.fixpointIterator;
import fr.fluxmedia.transmorpher.stdlib.fileIterator;
import fr.fluxmedia.transmorpher.stdlib.integerIterator;

import fr.fluxmedia.transmorpher.stdlib.writefile;


// Imported java.io classes
import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
// Imported TraX classes
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.sax.SAXResult;
import javax.xml.transform.sax.TransformerHandler;
// Imported SAX classes
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 *  This class represents a repeat process.
 *  A Loop can accept several iterator of different types.
 *
 *@author     Laurent.Tardif@inrialpes.fr
 *@author     Fabien.Triolet@inrialpes.fr
 *@created    5 aot 2002
 *@since      jdk 1.3 && SAX 2.0
 */
public final class TLoop extends TProcessComposite {

	/**Controller Handler */
	LoopManagerHandler managers[];

	/**In ports of this process */
	String iListLocalIn[] = null;

	/**Out ports of this process */
	String iListLocalOut[] = null;

	/**Buf ports of this process */
	String iListLocalBuf[] = null;

	/**List of inner processes */
	Vector listOfProcess = null;

	/**  Buf name */
	String[] nameBuf = null;

	/**  Buf ports of this process */
	XML_Port[] iListBuf = null;

	/**  true if it's first loop */
	boolean isFirstLoop = true;

	/**  flag for iterators, if one of them is finished then its value is true */
	boolean stop = false;

	/**  number of call to manage method */
	int numberOfManagers = 0;

	/**  counter of loop. */
	int loopCounter = 0;

	/**  List of iterator */
	Vector iterators;

	/**  List of inner serializer */
	Vector listOfSerializer;

	/**  the list of PortProcess buf */
	protected Hashtable listOfProcessBuf = null;

	/**
	 *  The constructor
	 *
	 *@param  pIn                                    in port names
	 *@param  pOut                                   out port names
	 *@param  pBuf                                   buffer port names
	 *@param  pParam                                 Description of the Parameter
	 *@param  staticAttributes                       Description of the Parameter
	 *@exception  TransformerException               Description of the Exception
	 *@exception  TransformerConfigurationException  Description of the Exception
	 *@exception  SAXException                       Any SAX exception, possibly wrapping another exception.
	 *@exception  IOException                        Description of the Exception
	 */
	public TLoop(String[] pIn, String[] pOut, String[] pBuf, Parameters pParam, StringParameters staticAttributes)
			 throws TransformerException,
			TransformerConfigurationException,
			SAXException,
			IOException {
		super(pIn, pOut, pParam);
		// managers creation.
		managers = new LoopManagerHandler[nameIn.length];
		for (int i = 0; i < nameIn.length; i++) {
			managers[i] = new LoopManagerHandler(this);
		}
		listOfProcess = new Vector();
		iterators = new Vector();
		listOfSerializer = new Vector();
		listOfProcessBuf = new Hashtable();
		iListBuf = new XML_Port[pBuf.length];
		nameBuf = pBuf;
		// ports creation
		innerGeneratePort(nameIn, nameOut, nameBuf);
	}//end constructor

	/**
	 *  links ports and creates feedbacks.
	 *
	 *@param  process  The current process to link to the others
	 */
	public void bindPorts(TProcess process) {
		//store process in order to reset them at the next loop.
		if (isFirstLoop) {
			listOfProcess.add(process);
		}
		process.generatePort();
		for (int i = 0; i < process.getNameOut().length; i++) {
			if (process.getNameOut(i).equals("_null_")) {
				this.addChannel(new TChannel("_null_", process, null));
			} else {
				listOfProcessOut.put(process.getNameOut(i), process);
				TProcess isSameIn = (TProcess) listOfProcessIn.get(process.getNameOut(i));
				TProcess isBuf = (TProcess) listOfProcessBuf.get(process.getNameOut(i));
				if ((isSameIn != null)) {
					this.addChannel(new TChannel(process.getNameOut(i), process, isSameIn));
					listOfProcessIn.remove(process.getNameOut(i));
					listOfProcessOut.remove(process.getNameOut(i));
				} else {// may be a composite port or feedback
					for (int i1 = 0; i1 < iListLocalOut.length; i1++) {
						if ((process.getNameOut(i).equals(iListLocalOut[i1])) && (isBuf != null)) {
							//creates feedback
							int position = 0;
							for (int j = 0; j < nameBuf.length; j++) {
								if (nameBuf[j].equals(process.getNameOut(i))) {
									position = j;
								}
							}
							XML_Port port1 = new XML_Port(process.getNameOut(i), this, managers[position].buffer, position, XML_Port.IN);
							try {
								XML_Port.link(process.getOut(i), port1);
								listOfProcessOut.remove(process.getNameOut(i));
							} catch (Exception e) {
								e.printStackTrace();
							}
							XML_Port port2 = new XML_Port(process.getNameOut(i), this, this, position, XML_Port.OUT);
							iListOut[position] = port2;
						} //end if
						else if (process.getNameOut(i).equals(iListLocalOut[i1])) {
							//composite port
							iListOut[i1] = process.getOut(i);
						}
					}//end for
				}//end if
			}//end if
		}//end for

		for (int i = 0; i < process.getNameIn().length; i++) {
			listOfProcessIn.put(process.getNameIn(i), process);
			TProcess isSameOut = (TProcess) listOfProcessOut.get(process.getNameIn(i));
			TProcess isBuf = (TProcess) listOfProcessBuf.get(process.getNameIn(i));
			if (isSameOut != null) {
				if (isBuf == null) {
					this.addChannel(new TChannel(process.getNameIn(i), isSameOut, process));
					listOfProcessIn.remove(process.getNameIn(i));
					listOfProcessOut.remove(process.getNameIn(i));
				} else {
					//creates feedback
					int position = 0;
					for (int j = 0; j < nameBuf.length; j++) {
						if (nameBuf[j].equals(process.getNameIn(i))) {
							position = j;
						}
					}
					System.out.println("Position in " + position);
					XML_Port port1 = new XML_Port(process.getNameIn(i), this, managers[position].buffer, position, XML_Port.IN);
					XML_Port port2;
					try {
						XML_Port.link(isSameOut.getOut(i), port1);
						if (isFirstLoop) {
							port2 = new XML_Port(process.getNameIn(i), this, this, position, XML_Port.OUT);
							XML_Port.link(port2, process.getIn(i));
						}
						listOfProcessIn.remove(process.getNameIn(i));
					} catch (Exception e) {
						e.printStackTrace();
					}
				}
			} else {// may be a composite port
				for (int i1 = 0; i1 < iListLocalIn.length; i1++) {
					if (process.getNameIn(i).equals(iListLocalIn[i1])) {
						managers[i1].loopHandler.setInnerHandler((ContentHandler) process.getIn(i).getContent());
						XML_Port port = new XML_Port(process.getNameIn(i), this, managers[i1], i1, XML_Port.IN);
						iListIn[i1] = port;
					}//end if
				}// end for
			}//end if
		}// end for
	}

	/**
	 *  To setup the Loop manager, the ports of this Loop have to be created before inner
	 *  process ports.
	 * In the case of TLoop local names and global names are the same.
	 *
	 *@param  locIn   local name of in ports
	 *@param  locOut  local name of out ports
	 *@param  locBuf  local name of buf ports
	 */
	public void innerGeneratePort(String[] locIn, String[] locOut, String[] locBuf) {
		//System.out.println("inner generate ports TLoop");
		iListLocalIn = locIn;
		iListLocalOut = locOut;
		iListLocalBuf = locBuf;
		for (int i = 0; i < nameIn.length; i++) {
			String name = locIn[i];
			iListIn[i] = new XML_Port(name, this, i, XML_Port.IN);
			listOfProcessIn.put(name, this);
		}//end for
		for (int i = 0; i < nameOut.length; i++) {
			String name = locOut[i];
			iListOut[i] = new XML_Port(name, this, i, XML_Port.OUT);
			listOfProcessOut.put(name, this);
		}//end for
		for (int i = 0; i < nameBuf.length; i++) {
			String name = locBuf[i];
			iListBuf[i] = new XML_Port(name, this, managers[i], i, XML_Port.IN);
			listOfProcessBuf.put(name, this);
		}//end for

	}

	/**
	 *  Generates all the process ports.In this case, nothing to do , ports have
	 *  been generated before.
	 */
	public void generatePort() {
	}//end proc

	/**
	 *  Initializes all inner processes for a new loop. Due to a xalan bug,
	 *  TransformerHandlerImpl can be use only once. So we have to create a new one
	 *  at each loop.
	 */
	public void reset() {
		listOfChannel = new Hashtable();
		isFirstLoop = false;
		for (int i = 0; i < listOfProcess.size(); i++) {
			TProcess process = (TProcess) listOfProcess.elementAt(i);
			process.reset();
			bindPorts(process);
		}//end for
	}//end reset

	/**
	 *  Sets one of the output handler of this loop with the content handler of
	 *  the following process.
	 *
	 *@param  c  the next content handler.
	 *@param  i  manager position.
	 */
	public void setContentHandler(ContentHandler c, int i) {
		managers[i].loopHandler.setOutputHandler((ContentHandler) c);
	}

	/**
	 *  Adds an Iterator to the Loop. If the iterator to add is an instance of fixpointIterator, TestHandler are used in each 
	 *  managers of this loop instead of LoopHandler.
	 *
	 *@param  iterator  The iterator to be added to this Loop
	 */
	public void addIterator(TIterator iterator) {
		//System.out.println("ADDED ITERATOR "+iterator);
		iterators.add(iterator);
		if (iterator instanceof fixpointIterator) {
			for (int i = 0; i < managers.length; i++) {
				managers[i].setTestHandler();
			}
		}
	}

	/**
	 *  Adds a process to the list of process of the TLoop object
	 *
	 *@param  process  The process to be added to the list of process
	 */
	public void addProcess(TProcess process) {
		super.addProcess(process);
		if (process instanceof TSerializer) {
			this.addSerializer(process);
		}
	}

	/**
	 *  Adds a process (TSerializer) to the list of serializer of the TLoop object
	 *
	 *@param  process  The process to be added to the list of serializer
	 */
	public void addSerializer(TProcess process) {
		listOfSerializer.add(process);
	}
	 
	 private boolean isReady(){
		 boolean test = false;
		 
		 numberOfManagers++;
		 if (numberOfManagers == managers.length) 
			 test=true;
		return test;
	}
		 
	/**
	 *  Manages the loop. When all the iterators have called this methods and if
	 *  they all have a next element , a new loop is started . If one of them is
	 *  finished, loop is ended .
	 */
	public void manage() {
		String fileName="";
		String  number="";
		try {//maybe try should not be here.
			/* numberOfManagers++;
			//waiting for end of all loop.
			if (numberOfManagers == managers.length) { */
			if (isReady()){
				//all loop have called this method and are ready to start a new loop. So now ,It's time to test iterators values
				for (int i = 0; i < iterators.size(); i++) {
					TIterator iterator = (TIterator) iterators.elementAt(i);
					if (!(iterator.hasMoreElements())) {
						stop = true;
					} else {
						if (!(iterator instanceof fixpointIterator)){
							if (iterator instanceof integerIterator){
								number=(String)iterator.nextElement();
							}
							if (iterator instanceof fileIterator){
								fileName=(String)iterator.nextElement();
							}
						}
						 /* if (iterator instanceof fileIterator){
							fileName=(String)iterator.nextElement();
						}  */
						else
							iterator.nextElement();
					}
				}
				// testing difference between last input and output. If no difference, stop loop.
				int counter = 0;
				for (int i = 0; i < managers.length; i++) {
					if (managers[i].loopHandler instanceof TestHandler) {
						if (!(((TestHandler) managers[i].loopHandler).isDiff())) {
							counter ++;
						}
						if(counter==managers.length)
							stop=true;
					}
				}
				if (!stop) {// each managers starts a new loop
					loopCounter++;
					numberOfManagers = 0;
					if (loopCounter != 1) {
						this.reset();
					}
					for (int j=0;j<listOfSerializer.size();j++){
						for (int i = 0; i < iterators.size(); i++) {
							TIterator iterator = (TIterator) iterators.elementAt(i);
							String n = fileName;
							// System.out.println("SERIALIZER :"+n);
							if (iterator instanceof integerIterator)
								((TSerializer)listOfSerializer.get(j)).setParameter(((integerIterator)iterator).getName(),number);
							if (iterator instanceof fileIterator)
								((TSerializer)listOfSerializer.get(j)).setParameter(((fileIterator)iterator).getName(),n.substring(fileName.lastIndexOf('/')+1));
						}
						((TSerializer)listOfSerializer.get(j)).setOutputStream();

					}
					for (int i = 0; i < managers.length; i++) {
						//System.out.println("NEW LOOP " + loopCounter);
						managers[i].loopHandler.newLoop();
					}					
					for (int i=0;i<listOfGenerator.size();i++){
						//System.out.println("GENERATOR :"+fileName);
						((TReader)listOfGenerator.get(i)).execProcess(fileName);
					}
					for (int i = 0; i < managers.length; i++) {
						//System.out.println("FLUSH BUFFER ");
						managers[i].buffer.flushBuffer();
					}
				} else {// end loop
					//System.out.println("END LOOP");
					for (int i = 0; i < managers.length; i++) {
						managers[i].loopHandler.endLoop();
					}
				}
			}
		} catch (Exception e) {
			System.out.println("[TLoop manage()] "+e);
			e.printStackTrace();
		}
	}
}// end class

/**
 *  This is the controller handler for a TLoop process
 *
 *@author     fabien.triolet@inrialpes.fr
 *@created    5 aot 2002
 */
final class LoopManagerHandler implements ContentHandler {

	/**  the owner */
	TLoop iOwner = null;

	/**  the loop handler to manage */
	LoopHandler loopHandler;
	//TestHandler loopHandler;

	/** a buffer for the output of inner process of the loop */
	BufferingHandler buffer;

	/**  number of loop */
	int nbLoop;

	/**
	 *  Constructs a new LoopManagerHandler and sets its owner *
	 *
	 *@param  loop  - owner
	 */
	public LoopManagerHandler(TLoop loop) {
		setOwner(loop);
		nbLoop = 0;
		loopHandler = new LoopHandler();
		buffer = new BufferingHandler(this);
	}

	/** 
	* Changes loop handler to test handler.
	*/
	void setTestHandler() {
		loopHandler = null;
		loopHandler = new TestHandler();
	}

	/**
	 *  Sets owner of this LoopManagerHandler
	 *
	 *@param  loop  - owner
	 */
	void setOwner(TLoop loop) {
		iOwner = loop;
	}

	/**
	 *  The startElement command
	 *
	 *@param  ns                The Namespace URI.
	 *@param  localName         The local name without prefix.
	 *@param  name              The local name with prefix.
	 *@param  atts              The attributes attached to the element.
	 *@exception  SAXException  Any SAX exception, possibly wrapping another exception.
	 */
	public final void startElement(String ns, String localName, String name, Attributes atts) throws SAXException {
		//System.out.println("LoopManagerHandler startElement "+name);
		loopHandler.startElement(ns, localName, name, atts);
	}

	/**
	 *  The endElement command
	 *
	 *@param  ns                The Namespace URI.
	 *@param  localName         The local name without prefix.
	 *@param  name              The local name with prefix.
	 *@exception  SAXException  Any SAX exception, possibly wrapping another exception.
	 */
	public final void endElement(String ns, String localName, String name) throws SAXException {
		//System.out.println(name+"/>");
		loopHandler.endElement(ns, localName, name);
	}

	/**
	 *  The startDocument command
	 *
	 *@exception  SAXException  Any SAX exception, possibly wrapping another exception.
	 */
	public final void startDocument() throws SAXException {
		//System.out.println("LoopManagerHandler startDocument");
		loopHandler.startDocument();
	}

	/**
	 *  The setDocumentLocator command
	 *
	 *@param  locator  An object that can return the location of any SAX document event.
	 */
	public final void setDocumentLocator(Locator locator) {
		loopHandler.setDocumentLocator(locator);
	}

	/**
	 *  The endDocument command
	 *
	 *@exception  SAXException  Any SAX exception, possibly wrapping another exception.
	 */
	public final void endDocument() throws SAXException {
		//System.out.println("LoopManagerHandler endDocument");
		loopHandler.endDocument();
		iOwner.manage();
	}

	/**
	 *  The skippedEntity command
	 *
	 *@param  name              The name of the skipped entity. If it is a parameter entity, the name will begin with '%', and if it is the external DTD subset, it will be the string "[dtd]".
	 *@exception  SAXException  Any SAX exception, possibly wrapping another exception.
	 */
	public final void skippedEntity(java.lang.String name) throws SAXException {
		loopHandler.skippedEntity(name);
	}

	/**
	 *  The processingInstruction command
	 *
	 *@param  target            The processing instruction target.
	 *@param  data              The processing instruction data, or null if none was supplied. The data does not include any whitespace separating it from the target.
	 *@exception  SAXException  Any SAX exception, possibly wrapping another exception.
	 */
	public final void processingInstruction(java.lang.String target, java.lang.String data) throws SAXException {
		loopHandler.processingInstruction(target, data);
	}

	/**
	 *  The ignorableWhitespace command
	 *
	 *@param  ch                The characters from the XML document.
	 *@param  start             The start position in the array.
	 *@param  length            The number of characters to read from the array.
	 *@exception  SAXException  Any SAX exception, possibly wrapping another exception.
	 */
	public final void ignorableWhitespace(char[] ch, int start, int length) throws SAXException {
		loopHandler.ignorableWhitespace(ch, start, length);
	}

	/**
	 *  The characters command
	 *
	 *@param  ch                The characters from the XML document.
	 *@param  start             The start position in the array.
	 *@param  length            The number of characters to read from the array.
	 *@exception  SAXException  Any SAX exception, possibly wrapping another exception.
	 */
	public final void characters(char[] ch, int start, int length) throws SAXException {
		/* char[] tmp = new char[length + start];
		for (int i = start; i < length + start; i++) {
			tmp[i] = ch[i];
		} */
		loopHandler.characters(ch, start, length);
	}

	/**
	 *  The startPrefixMapping command
	 *
	 *@param  prefix            The prefix that was being mapping.
	 *@param  uri               The Namespace URI the prefix is mapped to.
	 *@exception  SAXException  Any SAX exception, possibly wrapping another exception.
	 */
	public final void startPrefixMapping(java.lang.String prefix, java.lang.String uri) throws SAXException {
		loopHandler.startPrefixMapping(prefix, uri);
	}

	/**
	 *  The endPrefixMapping command
	 *
	 *@param  prefix            The prefix that was being mapping.
	 *@exception  SAXException  Any SAX exception, possibly wrapping another exception.
	 */
	public final void endPrefixMapping(java.lang.String prefix) throws SAXException {
		loopHandler.endPrefixMapping(prefix);
	}
}

