/**
 * $Id: TProcessComposite.java,v 1.25 2002/09/24 06:23:11 triolet Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001-2002 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/**
  *
  * This class allow to build a composite process
  *
  * @author Laurent Tardif (laurent.tardif@inrialpes.fr)
  * @since jdk 1.2 / SAX 2.0
  * @see fr.fluxmedia.transmorpher.TProcessFactory.XML_Port
*/

package fr.fluxmedia.transmorpher.TProcessFactory ;

import fr.fluxmedia.transmorpher.Utils.Parameters;
import fr.fluxmedia.transmorpher.Files.*;

// Imported JAXP classes

import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerConfigurationException;


// Imported SAX classes

import org.xml.sax.SAXException;

// Imported JAVA classes

import java.io.IOException;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Vector;


public abstract class TProcessComposite extends TProcess_Impl {
    
  /** The max number of process in a composite */
  public static int NB_PROCESS_RISE = 200;
    
   /** the list of the TProcessBasic (except the TApplyProcess) of
        our TProcessComposite. May contain no element.*/
  protected Vector listOfProcessBasic = null;
    
  /** the list of the TChannel of our TProcessComposite. May contain no element.*/
  protected Hashtable listOfChannel = null;

  /** the list of the Generator of our transmorpher. May contain no element.*/
  protected Vector listOfGenerator = null;

  /** the list of PortProcess In */
  protected Hashtable listOfProcessIn = null;

  /** the list of PortProcess Out */
  protected Hashtable listOfProcessOut = null;
    
  /** the TApplyProcess' list */
  protected Vector listOfApplyProcess = null;
    
  /** The constructor : build a Composite process with pNbIn in Port, and pNbOut Out Port  */
  public TProcessComposite(String[] pIn, String[] pOut, Parameters p)
    throws TransformerException, TransformerConfigurationException,
           SAXException, IOException  {
  
    super(pIn,pOut);
    // JE: Maybe this should go even upper...
    setParameters( p );
    listOfProcessBasic= new Vector(NB_PROCESS_RISE*5);
    listOfChannel = new Hashtable(NB_PROCESS_RISE*5);
    listOfGenerator = new Vector(NB_PROCESS_RISE*5);
    listOfProcessIn=new Hashtable(NB_PROCESS_RISE*5);
    listOfProcessOut=new Hashtable(NB_PROCESS_RISE*5);
    listOfApplyProcess = new Vector(NB_PROCESS_RISE*5);
  }

   
    public void addChannel(TChannel channel){
	if (listOfChannel.get(channel.getName())== null) {
	    listOfChannel.put(channel.getName(),channel);
	} else {
	    System.out.println("Error: This Channel already Exist!!");
	    System.exit(1);
	}
    } //END PROC
      
    public TChannel getChannel(String name){
	return  (TChannel)listOfChannel.get(name);
    }
    
    public void addGenerator(TProcess generator){
	// before to add this generator is this hashtable, it have been add in the list of Process.
	// So this generator have a single id
	listOfGenerator.add(generator);
    }
    
    public Vector getGenerator(){
	return listOfGenerator;
    }
    
  public Hashtable getListOfProcessIn(){
    return listOfProcessIn;
  }
    
  public Hashtable getListOfProcessOut(){
    return listOfProcessOut;
  }
    
  public void addApplyProcess(TProcess process){
    listOfApplyProcess.add(process);
  }
    
  public void addProcess(TProcess process){
      
      //System.out.println("process :" + process);
      process.setFatherName(getName());
      
      if(process instanceof TApplyProcess) {
	  addApplyProcess(process);
      } else {
	  listOfProcessBasic.add(process);
      } //end if
      if (process instanceof TReader)  this.addGenerator(process);
  } //end proc
    
    public Vector getApplyProcess(){
	return listOfApplyProcess;
    }
    
  public Vector getProcessBasic(){
    return listOfProcessBasic;
  }

  public Vector getAllProcess(){
    Vector allProcess = new Vector(listOfProcessBasic);
    allProcess.addAll(listOfApplyProcess);
    return allProcess;
  }

    public final void bindParameters(Parameters p){
	// bind its parameters to those of the calling process
	p.bindCallerParameters( iListOfParameters );
	// bind these parameters to the processes it calls

	for (Enumeration e = listOfProcessBasic.elements() ; e.hasMoreElements() ;) {
		TProcess process=(TProcess)e.nextElement();
	    process.bindParameters(iListOfParameters);
	}//end for
	for(Enumeration e = listOfApplyProcess.elements() ; e.hasMoreElements() ;) {
	    ((TProcess)e.nextElement()).bindParameters(iListOfParameters);
	} //end for
	bindPorts();
    }
 
    public final void bindPorts(){
	for (Enumeration e = listOfProcessBasic.elements() ; e.hasMoreElements() ;) {
	    bindPorts((TProcess)e.nextElement());
	} //end for
	for (Enumeration e = listOfApplyProcess.elements() ; e.hasMoreElements() ;) {
	    bindPorts((TProcess)e.nextElement());
	} //end for
    }
 
    public void bindPorts(TProcess process){
    process.generatePort() ;
   
   // System.out.println("debut for " + process);
    for(int i=0 ; i < process.getNameOut().length ; i++) {
	if(process.getNameOut(i).equals("_null_")) {
	    this.addChannel(new TChannel("_null_",process,null));
	} else {
	    listOfProcessOut.put(process.getNameOut(i),process);
	    TProcess isSameIn = (TProcess)listOfProcessIn.get(process.getNameOut(i));
	    if(isSameIn!=null){
		this.addChannel(new TChannel(process.getNameOut(i),process,isSameIn));
		listOfProcessIn.remove(process.getNameOut(i));
		listOfProcessOut.remove(process.getNameOut(i));
	    }//end if
	} //end if
    }//end for
    //System.out.println("fin for");
  
    for(int i=0 ; i < process.getNameIn().length ; i++) {
	//System.out.println(process.getNameIn(i).getCompositeName());
	listOfProcessIn.put(process.getNameIn(i),process);
        
	TProcess isSameOut =(TProcess)listOfProcessOut.get(process.getNameIn(i));
        
	if(isSameOut!=null){
	    this.addChannel(new TChannel(process.getNameIn(i),isSameOut,process));
	    listOfProcessIn.remove(process.getNameIn(i));
	    listOfProcessOut.remove(process.getNameIn(i));
	} //end if
    } //end for
    }
 
  /** Call the execution of the process */
  public final void execProcess(boolean useThread)
  // JE: it is certainly time to open stdin and stdout if it is necessary
    throws SAXException,IOException {
      for (Enumeration e = listOfGenerator.elements() ; e.hasMoreElements() ;) {
	  TReader reader = (TReader)e.nextElement();
	  reader.setThread(useThread);
	  reader.execProcess();
      } //end for
  } //end proc
    
  public void showChannel(){
    for (Enumeration e = listOfChannel.elements() ; e.hasMoreElements() ;) {
      TChannel channel = (TChannel)e.nextElement();
      System.out.println("channel : "+ channel.getName());
      System.out.println("transmitter : "+((TProcess)channel.getIn()).getName());
      System.out.println("receiver : "+((TProcess)channel.getOut()).getName());
      System.out.println("");
    } //end for
  } //end proc
     
 
  public void generatePort(){
    if(nameFather == null) {
      for(int i = 0; i<nameIn.length;i++){
        iListIn [i] = new XML_Port(nameIn[i],this,0,XML_Port.IN);
      }
      for(int i = 0; i<nameOut.length;i++){
        iListOut[i] = new XML_Port(nameOut[i],this,0,XML_Port.OUT);
      }
    } else {
      for(int i = 0; i<nameIn.length;i++){
        iListIn [i] = new XML_Port(nameFather+"-"+nameIn[i],this,0,XML_Port.IN);
      }
      for(int i = 0; i<nameOut.length;i++){
        iListOut[i] = new XML_Port(nameFather+"-"+nameOut[i],this,0,XML_Port.OUT);
      } //end for
    } //end if
  } //end proc


} //end class
