/**
 * $Id: readfile.java,v 1.9 2002/10/16 08:11:11 triolet Exp $
 *
 * Transmorpher
 *
 * Copyright (C) 2001-2002 Fluxmedia and INRIA Rhne-Alpes.
 *
 * http://www.fluxmedia.fr - http://transmorpher.inrialpes.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/**
 * This class provide a basic process which is
 * able to read an XML file
 *
 * @author Laurent.Tardif@inrialpes.fr
 * @since jdk 1.3 && SAX 2.0
 */


package fr.fluxmedia.transmorpher.stdlib ;

// Imported SAX classes

import fr.fluxmedia.transmorpher.TProcessFactory.*;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.InputSource;

// Imported Java classes

import java.io.IOException;
import java.io.InputStream;
import java.io.File;
import java.io.*;
import java.net.*;
import java.util.Hashtable;
import java.lang.Runnable ;

// Imported Transmorpher classes

import fr.fluxmedia.transmorpher.Utils.Parameters ;
import fr.fluxmedia.transmorpher.Utils.StringParameters;
import fr.fluxmedia.transmorpher.Utils.SystemResources;
import fr.fluxmedia.transmorpher.Files.ReadWrite;

import javax.xml.parsers.SAXParserFactory;

/**
 * read a file which coordinates (local path or URL) is give in the
 * file parameter of the generate call, parse it and generate a SAX event
 * flow to the next handler
 */

public final class readfile extends TReader {
    
    /** The file name whish describe the file to read */
    String iFilename = null;
    InputSource standardInput = null;
    
    public readfile(String[] pOut, Parameters pParam,StringParameters pStaticAttributes) throws SAXException, IOException   {
	
	super(pOut,pParam,pStaticAttributes);
	try {
	   
	   SAXParserFactory parserFactory = SAXParserFactory.newInstance();
	    
	    parserFactory.setFeature(NAMESPACES_FEATURE_ID, DEFAULT_NAMESPACES);
	    parserFactory.setFeature(NAMESPACES_PREFIXES_FEATURE_ID, DEFAULT_NAMESPACES_PREFIXES);
	    parserFactory.setFeature(VALIDATION_FEATURE_ID, DEFAULT_VALIDATION);
	    parserFactory.setFeature(SCHEMA_VALIDATION_FEATURE_ID, DEFAULT_SCHEMA_VALIDATION); 
	     
	    iReader = (parserFactory.newSAXParser()).getXMLReader();	    
	    //System.out.println(iReader);
	    
	} catch(Exception e) {
	    System.out.println("[readfile]Exception :"+e);
	    System.out.println("[readfile] "+SystemResources.getProperty("XML_READER"));
	} //end try
	//if (isThread) myThread = new Thread(this);
    }
   
    /** Exec the process :parse a file or an URL content*/
    public void execProcess() throws IOException,SAXException {
	iFilename = (String)getParameter("file");
	if (isThread) {
	    myThread.start();
	} else {
		read();
	}	
    } //end proc
    	
	public void read()throws IOException,SAXException{
		try{
		InputSource inSource;
		if (iFilename !=null){
			if (iFilename.startsWith("http://")){
				URL url = new URL(iFilename);
				URLConnection urlConnect = url.openConnection();
				InputStream inStream = urlConnect.getInputStream();
				inSource = new InputSource(inStream);
				inSource.setSystemId(iFilename);
				iReader.parse(inSource);
			}
			else{
				File file = new File(iFilename);
				 inSource = new InputSource(new FileInputStream(file));
				inSource.setSystemId(file.toURL().toString()); 
				iReader.parse(inSource);
			}
		}
		else
			iReader.parse(standardInput);
		}
		catch(IOException e){
			System.err.println("File : "+iFilename +" can't be found.");
			iReader.getContentHandler().startDocument();
			iReader.getContentHandler().endDocument();
		}
	}
	
	/**
	 *  Begins the parse of a file. This method is called inside a repeat . At each loop, file is changed. 
	 * @Param file The file to parse.
	 */
        public void execProcess(String file) throws IOException,SAXException {
		iFilename = file;
		try{
		if(iFilename != null) {
			iReader.parse(iFilename);
		} else {
			iReader.parse(standardInput);
		} //end if
		}
		catch(SAXException e){
			System.out.println("[readfile] "+ e+" "+file);
			e.printStackTrace();
		}
	}
  /* public void generatePort(){
    iListOut[0] = new XML_Port(nameOut[0],this,(XMLReader)iReader,0,XML_Port.OUT);
  } */

  
  public final void run() {
    //System.out.println("euh .. je fais qqc ?");
    if (!isRunning) {
      try {
        //System.out.println("debut du parse  2"+iFilename);
	read();
        //System.err.println("Fin du parse  2");
        isRunning=true;
        //myThread.interrupt();
      } catch (Exception e) {
        System.err.println("Fin du parse 2 !erreur dans le thread" + iFilename);
        e.printStackTrace();
        isRunning=true;
      } //end try
    } //fin if
  } //end run
}
